#ifndef TEST
#include <iostream>
#include <cstdlib>
using namespace std;

#include "100pris.h"

/*
 * 100pris
 *
 *   Simulate runs of the 100 prisoners and a lightbulb scenario and
 * report a statistical summary of the results
 */

int main(int argc, char * argv[]) {
	
	// check for proper usage
	if (argc < 2 || argc > 3 || atoi(argv[1]) <= 0) {
		cerr << "Usage: " << argv[0] << " runs [seed]" << endl;
		cerr << "  runs -- the number of simulation runs to perform" << endl;
		cerr << "  seed -- a seed value for the random number generator" << endl;
		cerr << endl;
		return EXIT_FAILURE;
	}
	
	// get commandline parameters
	int runs = atoi(argv[1]);
	unsigned int seed = (argc > 2) ? (unsigned int)atol(argv[2]) : 0;

	// perform the simulation runs
	Simulator sim(100, 0);
	Statistics stats = sim.simulate(runs);
	
	// report results
	if (runs == 1) {
		cout << stats.getMean() << " Days" << endl;
	} else {
		cout << "  Samples: " << stats.getSamples() << " Runs" << endl;
		cout << "     Mean: " << stats.getMean() << " Days" << endl;
		cout << "Std. Dev.: " << stats.getStdDev() << " Days" <<  endl;
	}
	
	return EXIT_SUCCESS;
}


#endif
