#include <climits>
using namespace std;

#include "100pris.h"

/*
 * MersenneTwister
 *
 *   A utility class implementing a pseudorandom number generator of
 * sufficient quality for Monte Carlo simulations (unlike the rand()
 * library function which may show considerable artifacting).
 *
 *   Returns an integer uniformly distributed in 0 to max-1
 */

MersenneTwister::MersenneTwister(unsigned int max, unsigned int seed)
	: _max(max), _maxrand((UINT_MAX / max) * max)
{
	reseed(seed);
}
	
unsigned int MersenneTwister::rand() {
	int y;
	do {
		y = _state[_index];
		y ^= y >> 11;
		y ^= y << 7 & 0x9d2c5680;
		y ^= y << 15 & 0xefc60000;
		y ^= y >> 18;
	
		if (++_index == 624) _generate();
	} while (y >= _maxrand);
	
	return y % _max;
}

void MersenneTwister::reseed(unsigned int seed) {
	_state[0] = seed;
	for(int i = 1; i < 624; ++i) {
		_state[i] = 0x6c078965 * (_state[i-1] ^ (_state[i-1] >> 30)) + i;
	}
	// skip first few hundred-thousand values to let the state settle
	for(int i = 0; i < 624; ++i) _generate();
}

void MersenneTwister::_generate() {
	for(int i = 0; i < 227; ++i) {
		int y = _state[i] & 0x80000000 | _state[i+1] & 0x7FFFFFFF;
		_state[i] = _state[i + 397] ^ (y >> 1);
		if (y & 1) _state[i] ^= 0x9908b0df;
	}
	for(int i = 227; i < 623; ++i) {
		int y = _state[i] & 0x80000000 | _state[i+1] & 0x7FFFFFFF;
		_state[i] = _state[i - 227] ^ (y >> 1);
		if (y & 1) _state[i] ^= 0x9908b0df;
	}
	int y = _state[623] & 0x80000000 | _state[0] & 0x7FFFFFFF;
	_state[623] = _state[396] ^ (y >> 1);
	if (y & 1) _state[623] ^= 0x9908b0df;

	_index = 0;
}
