#ifndef TEST
#include "100pris.h"

/*
 * Prisoner
 *
 *   A prisoner together with their internal state.  The declaresVictory
 * function is the main decision point for all prisoner actions.
 */

Prisoner::Prisoner() :
	_intState()
{
}

bool Prisoner::declaresVictory(ExternalState & extState) {
	int round = SharedKnowledge::getCurrentRound(extState.getDay()),
		day = SharedKnowledge::getCurrentRoundDay(extState.getDay());

	if (round < SharedKnowledge::getNumBadges()) {
		// badge assignment round
		if (day == 1) {
			// first day of round
			if (extState.getLightbulb() != OFF) {
				// pick up orphaned badge assignment
				_intState.assignBadge(round - 1);
				if (round == 1) _intState.assignCrown();
				_intState.pickupTokens(SharedKnowledge::getRoundLength(round - 1) - 1);
				extState.setLightbulb(OFF);
			}
			
			if (_intState.hasTokens() || _intState.hasBadges()) {
				_intState.dropToken();
				extState.setLightbulb(ON);
			} else {
				_intState.assignBadge(round);
				if (round == 0) _intState.assignCrown();
				extState.setLightbulb(OFF);
			}

		} else if (day < SharedKnowledge::getRoundLength(round)) {
			// middle day of round
			if (extState.getLightbulb() != OFF) {
				if (_intState.hasTokens() || _intState.hasBadges()) {
					_intState.dropToken();
				} else {
					_intState.assignBadge(round);
					if (round == 0) _intState.assignCrown();
					_intState.pickupTokens(day - 1);
					extState.setLightbulb(OFF);
				}
			}

		} else {
			// last day of round
			if (extState.getLightbulb() != OFF && !_intState.hasBadges()) {
				_intState.assignBadge(round);
				if (round == 0) _intState.assignCrown();
				_intState.pickupTokens(day - 1);
				extState.setLightbulb(OFF);
			}
		}

	} else if (((round - SharedKnowledge::getNumBadges()) & 1) == 0) {
		// token-gathering round
		if (day == 1 && extState.getLightbulb() != OFF) {
			if (round == SharedKnowledge::getNumBadges()) {
				// pick up orpahned badge assignment
				_intState.assignBadge(round - 1);
				_intState.pickupTokens(SharedKnowledge::getRoundLength(round - 1) - 1);
			} else {
				// pick up orpahned filled badge
				_intState.pickupBadge();
			}
			extState.setLightbulb(OFF);
		}

		if (extState.getLightbulb() != OFF && _intState.hasBadges()) {
			_intState.pickupTokens(1);
			extState.setLightbulb(OFF);

		} else if (extState.getLightbulb() == OFF &&
			_intState.hasTokens() && !_intState.hasBadges())
		{
			_intState.dropToken();
			extState.setLightbulb(ON);
		}

	} else {
		// badge-gathering round
		if (day == 1 && extState.getLightbulb() != OFF) {
			// pick up orpahned token
			_intState.pickupTokens(1);
			extState.setLightbulb(OFF);
		}

		if (extState.getLightbulb() != OFF && _intState.hasCrown()) {
			_intState.pickupBadge();
			extState.setLightbulb(OFF);
			
		} else if (extState.getLightbulb() == OFF &&
			_intState.hasFilledBadges()&& !_intState.hasCrown())
		{
			_intState.dropBadge();
			extState.setLightbulb(ON);
		}
	}

	return _intState.victory();
}

void Prisoner::reset() {
	_intState.reset();
}


#endif
