#ifndef TEST
#include <cstdlib>
#include <iostream>
using namespace std;

#include "100pris.h"

/*
 * Simulator
 *
 *   The scenario simulator: simulates a single run or multiple runs of
 * the prisoners' decision algorithm to completion and counts the number
 * of days required
 */

Simulator::Simulator(int prisoners, int prng_seed)
	: _numPrisoners(prisoners), _prng(prisoners, prng_seed)
{
	_prisoners = new Prisoner[prisoners];
	_visited = new bool[prisoners];
}

Simulator::~Simulator() {
	delete[] _prisoners;
	delete[] _visited;
}

int Simulator::simulate() {
	int numVisited = 0;
	for(int i = 0; i < _numPrisoners; ++i) {
		_prisoners[i].reset();
		_visited[i] = false;
	}
	
	unsigned int p;
	ExternalState extState;
	do {
		extState.nextDay();
		p = _prng.rand();
		if (!_visited[p]) {
			_visited[p] = true;
			++numVisited;
		}
	} while (!_prisoners[p].declaresVictory(extState));

	if (numVisited < _numPrisoners) {
		cerr << "ERROR: Victory declared on day " << extState.getDay();
		cerr << " but only " << numVisited << " prisoners had visited!" << endl;
		exit(EXIT_FAILURE);
	}
	
	return extState.getDay();
}

Statistics Simulator::simulate(int runs) {
	Statistics stats;
	for(int i = 0; i < runs; ++i) {
		stats.addSample(simulate());
	}
	return stats;
}
	
void Simulator::reseed(int prng_seed) {
	_prng.reseed(prng_seed);
}


#endif
